/* $Id: bchart.java,v 1.61 2017/03/22 04:49:11 mit3 Exp $
/* bchart.java v0.4 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.text.DecimalFormat;

public class bchart extends Applet 
    implements ItemListener, ActionListener, AdjustmentListener,
               MouseListener, MouseMotionListener
{
    final int KEY_BOARD = 88;
    final int OCTAVE = 12;

    final int INHA = 0;
    final int CENT = 1;
    final int BEAT1 = 2;
    final int BEAT2 = 3;
    final int BEAT0 = 4;
    final int BEAT3 = 5;

    final int PARTIAL = 20;
    final int SP88 = 22;        // 1/22
    final int SP49 = 8;         // 1/8
    final int SA = KEY_BOARD-48-1;
    final float InitColor = 0.76f;
    final double SAQ = Math.pow(SP88/(double)SP49, 1.0/SA);
    final double RAD = Math.PI/180.0;
    final String[] Intvl =
    {"Oc","i2","J2","i3","J3","P4","O2","P5","i6","J6","i7","J7"};
    final String[] Graph =
    {"Inha.", "Cent", "Beat1", "Beat2", "Beat3", "Beat4"};
    final String[] OCTAVE_NAME =
    {"A2","A1","A","a","a1","a2","a3","a4"};
    final String[] KEY_NAME =
    {"A..","B..","H..","C..","Cis","D..","Es.","E..","F..","Fis","G..","Gis"};
    private static final long serialVersionUID = 1L;

    private Button[] btGraph;
    private Button btTune;
    private Button btPlus;
    private Button btMinus;
    private Button btWide;
    private Button btNarrow;
    private Button btUp;
    private Button btDown;
    private Button btBright;
    private Button btDark;
    private Button btColor;
    private Button btText;
    private Button btRev;
    private Button btLR;
    private Button btAbs;
    private Choice chPartial;
    private Scrollbar scrCent;
    private Scrollbar scrColor;
    private Label lbColor;
    private Checkbox[] cbIntvl;
    private TextField tfData;
    private TextArea taA;
    private CardLayout Card;
    private Panel deck;
    private Canvas can;
    private Panel PN, PE, PW, PS;

    private Graphics gi;
    private Image ig;
    private Font Fnt;
    private DecimalFormat df1, df0;

    private boolean TUNE_SW;
    private boolean MOUSE_SW = true;
    private boolean GT_SW;
    private boolean CB_SW; // btRev. for checkbox
    private boolean LR_SW; // ->,<-
    private boolean ABS_SW;
    private boolean COLOR_SW;
    private boolean[] INTV;
    private int W, H, W2, H2;
    private int KEY;
    private int FONT;
    private int GRPH;
    private int DOT;
    private int DSF;
    private int MOUSEX;
    private int WOUND;
    private int MULTI = 8;
    private int CENT_MAX = 40;
    private int INTN;
    private double DKEY;      // deltaKey
    private double BBY = 1.0; // beat
    private double WBY = 1.0; // wide
    private double CBY = 1.0; // cent
    private double COLOR_BY = 2.2;  // beat
    private double CENT_WIDTH = 50;
    private double BEAT_MAX = 20;
    private double CentTemp;
    private Color[] Col;

    private MakeInharmonicity MI;
    private TuningTriads TT;
    private Interval2 IN;

    public void init() {
        String fnt = getParameter("font");
        FONT = (fnt != null) ? Integer.parseInt(fnt) : 12;
        Fnt = new Font("SansSerif", Font.PLAIN, FONT);
        setFont(Fnt);
        df1 = new DecimalFormat("#0.0##");
	df0 = new DecimalFormat("#0.#");

        buildLayout();
        initSeting();
	initColor(InitColor);
    }
    public void buildLayout() {
        setBackground(Color.lightGray);
	setLayout(new BorderLayout());

        int num = Graph.length;
        PW = new Panel();
        PW.setLayout(new GridLayout(num+1,1,0,0));
        add(PW, "West");

        btGraph = new Button[num];
        for (int i = 0; i < num; i++) {
            btGraph[i] = new Button(Graph[i]);
            btGraph[i].addActionListener(this);
            PW.add(btGraph[i]);
        }

	btText = new Button("Graph");
        btText.addActionListener(this);
        PW.add(btText);
        //---

        PS = new Panel();
	PS.setLayout(new GridLayout(1,Intvl.length+1,0,0));
	add(PS, "South");

	INTV = new boolean[OCTAVE];
	cbIntvl = new Checkbox[Intvl.length];
	for (int i = 0; i < Intvl.length; i++) { // p4,p5,Oc
	    boolean sw = (i == 5 || i == 7 || i == 0) ? true : false;
	    INTV[i] = sw;
	    cbIntvl[i] = new Checkbox(Intvl[i], sw);
	    cbIntvl[i].addItemListener(this);
	    PS.add(cbIntvl[i]);
	}

	btRev = new Button("Rev.");
	btRev.addActionListener(this);
	PS.add(btRev);
        //---

        PN = new Panel();
	PN.setLayout(new GridLayout(2,1,0,0));
        add(PN, "North");

	Panel n0 = new Panel();
	n0.setLayout(new FlowLayout());
	PN.add(n0);

	Panel n1 = new Panel();
	n1.setLayout(new FlowLayout());
	PN.add(n1);

        tfData = new TextField(16);
        tfData.setEditable(false);
        n0.add(tfData);

	chPartial = new Choice();
        for (int i = 0; i < PARTIAL; i++)
            chPartial.addItem(""+(i+1));
        n0.add(chPartial);
        chPartial.select(7); // MULTI=8
        chPartial.addItemListener(this);

	btTune = new Button("Tune3");
	btTune.addActionListener(this);
        n0.add(btTune);

	btLR = new Button("->");
	btLR.addActionListener(this);
	n0.add(btLR);

	btAbs = new Button("abs.");
	btAbs.addActionListener(this);
	n0.add(btAbs);

	btColor = new Button("col.");
	btColor.addActionListener(this);
	n1.add(btColor);

	btBright = new Button("Brt.+");
	btBright.addActionListener(this);
	n1.add(btBright);

	btDark = new Button("Drk.-");
	btDark.addActionListener(this);
	n1.add(btDark);

        lbColor = new Label("BeatColor:"+InitColor, Label.RIGHT);
        n1.add(lbColor);

	scrColor = new Scrollbar(Scrollbar.HORIZONTAL,
				 (int)(InitColor*100),1,0,100);
        n1.add(scrColor);
        scrColor.addAdjustmentListener(this);
	//---

	PE = new Panel();
        PE.setLayout(new GridLayout(1,2,0,0));
        add(PE, "East");

	Panel e0 = new Panel();
	e0.setLayout(new GridLayout(1,1,0,0));
	PE.add(e0);

	Panel e1 = new Panel();
	e1.setLayout(new GridLayout(8,1,0,0));
	PE.add(e1);

	btUp = new Button("X");  // scrbar(cent)Ψ
	btUp.addActionListener(this);
        e1.add(btUp);

	btDown = new Button("/");
	btDown.addActionListener(this);
        e1.add(btDown);

	e1.add(new Label(""));

	btPlus = new Button("+"); // beatΨ
        btPlus.addActionListener(this);
        e1.add(btPlus);

        btMinus = new Button("-");
        btMinus.addActionListener(this);
        e1.add(btMinus);

	e1.add(new Label(""));

	btNarrow = new Button("<"); // wide(beat2)Ψ
	btNarrow.addActionListener(this);
	e1.add(btNarrow);

	btWide = new Button(">");
	btWide.addActionListener(this);
	e1.add(btWide);

	scrCent = new Scrollbar(Scrollbar.VERTICAL,50,10,0,110);
	e0.add(scrCent);
        scrCent.setBlockIncrement(10);
        scrCent.addAdjustmentListener(this);
        //---

        can = new myCanvas();

        can.addMouseListener(this);
        can.addMouseMotionListener(this);

        deck = new Panel();
        Card = new CardLayout();
        deck.setLayout(Card);

        Panel card1 = new Panel();
        card1.add(can);
        deck.add(card1, "card1");

        taA = new TextArea("",16,40,TextArea.SCROLLBARS_BOTH);
        taA.setFont(Fnt);
        taA.setEditable(false);

        ScrollPane card2 = new ScrollPane();
        card2.add(taA);
        deck.add(card2, "card2");

	add(deck, "Center");
    }
    public void initSeting() {
        MI = new MakeInharmonicity(); //0.000001, 0.0, 0.0);
	TT = new TuningTriads();
	TT.setInha(MI.getInha());
	TT.initCent();
	IN = new Interval2();

	KEY = 49-1;
	WOUND = MI.getWoundWire();
	INTN = 3; // p4,p5,Oc

	reset_tune();
	set_enabled();
    }
    public void initColor(float sft) {
        Col = new Color[PARTIAL];
        double dc = 1.0/PARTIAL;
        float c0;

        for (int i = 0; i < PARTIAL; i++) {
            c0 = (float)(dc*i+sft);
            if (c0 > 1.0)
                c0 -= 1.0;
            Col[i] = Color.getHSBColor(c0, 1.0f, 1.0f);
        }
    }
    public void start() {
        W = getCanvasD().width;
        H = getCanvasD().height;
	W2 = W/2;
        H2 = H/2;

        can.setSize(W, H);
        ig = createImage(W, H);
        gi = ig.getGraphics();
        gi.setFont(Fnt);

	write_text(); // ǽɽ
    }
    public Dimension getCanvasD() {
        Dimension dn = PN.getSize();
        Dimension de = PE.getSize();
        Dimension dw = PW.getSize();
	Dimension ds = PS.getSize();
        Dimension da = getSize();
        int cw = da.width-(de.width+dw.width);
        int ch = da.height-(dn.height+ds.height)-6;
        Dimension can = new Dimension(cw, ch);

        return can;
    }
    public void set_enabled() {
	boolean b2sw = (GRPH == BEAT2);
	btNarrow.setEnabled(b2sw);
	btWide.setEnabled(b2sw);

	boolean isw = (GRPH != INHA);
	btPlus.setEnabled(isw);
	btMinus.setEnabled(isw);
	scrCent.setEnabled(isw);

	boolean b0sw = (GRPH == BEAT0 || GRPH == BEAT3);
	btAbs.setEnabled(b0sw);

	boolean bsw = (GRPH == BEAT1 || b2sw || b0sw);
	btRev.setEnabled(bsw);
	btLR.setEnabled(bsw);
	btColor.setEnabled(bsw);

	boolean csw = (!COLOR_SW && bsw);
	btBright.setEnabled(csw);
	btDark.setEnabled(csw);
    }
    public void disp_sw() {
        if (GT_SW)
            re_write();
        else
            can.repaint();

	write_text();
    }
    //
    public void mouseMoved(MouseEvent me) {
	if (!MOUSE_SW) {
            MOUSEX = me.getX();
            mouse_check();

	    disp_sw();
        }
    }
    public void mouse_check() {
	KEY = (int)(MOUSEX/DKEY);

	if (KEY < 0)
	    KEY = 0;
	else if (KEY >= KEY_BOARD)
	    KEY = KEY_BOARD-1;

	set_scrCent();
    }
    public void set_scrCent() {
	scrCent.setValue(50); // reset

	CentTemp = TT.getCent(KEY);
    }
    public void mouseDragged(MouseEvent me) {}
    public void mousePressed(MouseEvent m) {
	MOUSE_SW = !MOUSE_SW;
    }
    public void mouseClicked(MouseEvent m) {}
    public void mouseReleased(MouseEvent m){}
    public void mouseEntered(MouseEvent m) {}
    public void mouseExited (MouseEvent m) {}
    //
    public void adjustmentValueChanged(AdjustmentEvent e) {
	if (e.getSource() == scrCent)
	    set_cent();

	if (e.getSource() == scrColor)
	    set_color();

	disp_sw();
    }
    public void set_color() {
        float co = (float)(scrColor.getValue()/100.0);
        lbColor.setText("BeatColor:"+co);
        initColor(co);
    }
    public void set_cent() {
	double ce = (50-scrCent.getValue())/CBY;

	TT.setCent(KEY, ce+CentTemp);
    }
    //
    public void itemStateChanged(ItemEvent e) {
	if (e.getSource() == chPartial)
	    set_partial();

	set_check_box();

	disp_sw();
    }
    public void set_check_box() {
	INTN = 0;
	for (int i = 0; i < Intvl.length; i++) { // checkbox
	    if (cbIntvl[i].getState()) {
		INTV[i] = true;
		INTN++;
	    } else
		INTV[i] = false;
	}
    }
    public void set_partial() {
        MULTI = Integer.parseInt(chPartial.getSelectedItem());
    }
    //
    public void actionPerformed(ActionEvent e) {
        for (int i = 0; i < Graph.length; i++) {
            if (e.getSource() == btGraph[i]) {
                set_graph(i);
                break;
            }
        }

	if (e.getSource() == btTune)
            reset_tune();

	if (e.getSource() == btPlus)
	    set_bby(true);
	if (e.getSource() == btMinus)
	    set_bby(false);

	if (e.getSource() == btText)
            disp_text();

	if (e.getSource() == btRev)
	    reset_checkbox();

	if (e.getSource() == btNarrow)
	    set_wide(false);
	if (e.getSource() == btWide)
	    set_wide(true);

	if (e.getSource() == btUp)
	    set_cent_by(true);
	if (e.getSource() == btDown)
	    set_cent_by(false);

	if (e.getSource() == btBright)
	    set_color(true);
	if (e.getSource() == btDark)
	    set_color(false);

	if (e.getSource() == btColor)
	    select_color();

	if (e.getSource() == btLR)
	    set_lr();

	if (e.getSource() == btAbs)
	    set_abs();

	set_enabled();
	disp_sw();
    }
    public void select_color() {
	COLOR_SW = !COLOR_SW;

	btColor.setLabel(COLOR_SW ? "COL." : "col.");
    }
    public void set_color(boolean sw) {
	double by = 0.5;
	COLOR_BY = (sw) ?  COLOR_BY+by : COLOR_BY-by;
	if (COLOR_BY < 0.1)
	    COLOR_BY = 0.1;
    }
    public void set_cent_by(boolean sw) {
	switch (GRPH) {
        case INHA:
	    break;
	case CENT:
	case BEAT1:
	case BEAT2:
	case BEAT0:
	case BEAT3:
	    CBY = (sw) ? CBY/2.0 : CBY*2.0;
	    if (CBY < 1.0)
		CBY = 1.0;
	    set_scrCent();
	    show_scrollbar();
	    break;
	default:
        }
    }
    public void show_scrollbar() {
	String by = dform0(50/CBY);
	showStatus("Scrollbar: -"+by+":"+by+"[cent]");
    }
    public void set_abs() {
	ABS_SW = !ABS_SW;

	btAbs.setLabel(ABS_SW ? "ABS." : "abs.");
    }
    public void set_lr() {
	LR_SW = !LR_SW;

	btLR.setLabel(LR_SW ? "<-" : "->");
    }
    public void set_wide(boolean sw) {
	switch (GRPH) {
	case BEAT2:
	    WBY = (sw) ? WBY/2.0 : WBY*2.0;
	    CENT_WIDTH = (sw) ? CENT_WIDTH*2.0 : CENT_WIDTH/2.0;
	    break;
	case INHA:
	case CENT:
	case BEAT1:
	case BEAT0:
	case BEAT3:
	    break;
	default:
        }
    }
    public void reset_checkbox() {
	CB_SW = !CB_SW;

	INTN = 0;
	for (int i = 0; i < Intvl.length; i++) {
	    cbIntvl[i].setState(CB_SW);
	    INTV[i] = CB_SW;
	    if (CB_SW)
		INTN++; 
	}
    }
    public void disp_text() {
        GT_SW = !GT_SW;

        btText.setLabel(GT_SW ? "Text " : "Graph");
        Card.next(deck);
    }
    public void set_bby(boolean sw) {
        switch (GRPH) {
        case INHA:
	    break;
	case CENT:
	    CENT_MAX = sw ? CENT_MAX*2 : CENT_MAX/2;
            if (CENT_MAX < 5)
                CENT_MAX = 5;
	    break;
	case BEAT1:
	case BEAT2:
	    BBY = (sw) ? BBY/2.0 : BBY*2.0;
	    break;
	case BEAT0:
	case BEAT3:
	    BEAT_MAX = (sw) ? BEAT_MAX*2.0 : BEAT_MAX/2.0;
	    if (BEAT_MAX < 1)
		BEAT_MAX = 1;
	    break;
	default:
        }
    }
    public void reset_tune() {
	set_tune();

	set_scrCent();
    }
    public void set_tune() {
	TUNE_SW = !TUNE_SW;

	if (TUNE_SW) {
	    TT.Tuning();
	} else {
	    TT.initCent();
	}
	btTune.setLabel(TUNE_SW ? "Clear" : "Tune3");
    }
    public void set_graph(int sw) {
        GRPH = sw;
    }
    //
    public void re_write() {
        switch (GRPH) {
        case INHA:
            write_inha();
            break;
        case CENT:
            write_cent();
            break;
        case BEAT1:
	case BEAT2:
	case BEAT0:
	case BEAT3:
	    write_beat();
	    break;
        default:
	    break;
        }
    }
    public void write_inha() {
        StringBuffer str = new StringBuffer(1024);
        str.append("Key#: [Hz]: Inharmonicity\n");
        str.append("-------------------------\n");

        double[] inha = MI.getInha();
        for (int i = 0; i < KEY_BOARD; i++) {
            str.append(getKeyname(i)+": ");
	    str.append(dform(TT.get_ifreq(i, 1))+": "); 
            str.append(dform(inha[i])+"\n");
        }

        taA.setText(str.toString());
    }
    public void write_cent() {
        StringBuffer str = new StringBuffer(1024);
        str.append("Key#: [Hz]: [cent]\n");
        str.append("------------------\n");

        for (int i = 0; i < KEY_BOARD; i++) {
            str.append(getKeyname(i)+": ");
	    str.append(dform(TT.get_ifreq(i, 1))+": ");
	    str.append(dform(TT.getCent(i))+"\n");
        }

        taA.setText(str.toString());
    }
    public void write_beat() {
        StringBuffer str = new StringBuffer(1024);

	str.append(getKeyname(KEY)+" ");
	str.append(dform(TT.get_ifreq(KEY, 1))+"[Hz]");
	for (int i = 0; i < OCTAVE; i++) {
	    if (!INTV[i])
		continue;

	    str.append(": "+IN.getInter(i));
	}
	str.append(":(Gain)\n");
	str.append(LR_SW ? "<<<---\n" : "--->>>\n");

	write_beat_sub(str);
    }
    public void write_beat_sub(StringBuffer str) {
	int start, end;
	double gin;
	Interv iv = new Interv();

	for (int j = 1; j <= MULTI; j++) {
	    str.append(j+" ");
	    for (int i = 0; i < OCTAVE; i++) {
		if (!INTV[i])
		    continue;

		iv.setInterv(IN.getInterval(i), LR_SW);

		if (LR_SW && KEY+iv.v < 0)
		    str.append(" ___ ");
		else if (!LR_SW && KEY+iv.v >= KEY_BOARD)
		    str.append(" --- ");
		else {
		    str.append(": ");
		    str.append(dform(getMbeats2(KEY, iv, j)));
		    gin = calc_sindo2(TT.get_ifreq(KEY, j*iv.l), j*iv.l,
				      TT.get_ifreq(KEY+iv.v, j*iv.h), j*iv.h);
		    str.append("("+dform(gin)+")");
		}
	    }
	    str.append("\n");
	}

	taA.setText(str.toString());
    }
    //
    public void write_text() {
	StringBuffer sb = new StringBuffer(40);

	sb.append(getKeyname(KEY)+": ");
	switch (GRPH) {
        case INHA:
	    sb.append(dform(MI.getInha(KEY)));
	    break;
        case CENT:
        case BEAT1:
	case BEAT2:
	case BEAT0:
	case BEAT3:
	    sb.append(dform(TT.getCent(KEY)));
	    sb.append("[cent]");
	    break;
        default:
            sb.append(getKeyname(KEY)+":");
	    break;
	}

	tfData.setText(sb.toString());
    }
    //
    public double log_base(double value) {
        return Math.log(value)/Math.log(10);
    }
    public String dform(double n) {
        return df1.format((float)n);
    }
    public String dform0(double n) {
        return df0.format((float)n);
    }
    public class Interv {
	private int l;
	private int h;
	private int v;

	public Interv() {
	    setInterv(0, 0, 0);
	}
	public Interv(int[] v) {
	    setInterv(v[0], v[1], v[2]);
	}
	public Interv(int a, int b, int c) {
	    setInterv(a, b, c);
	}
	public Interv(int[] v, boolean sw) {
	    setInterv(v[0], v[1], v[2], sw);
	}
	public Interv(int a, int b, int c, boolean sw) {
	    setInterv(a, b, c, sw);
	}
	public void setInterv(int a, int b, int c) {
	    l = a;
	    h = b;
	    v = c;
	}
	public void setInterv(int[] v, boolean sw) {
	    setInterv(v[0], v[1], v[2], sw);
	}
	public void setInterv(int a, int b, int c, boolean sw) {
	    if (sw) { // revers sw.
		h = a;
		l = b;
		v = -c;
	    } else {
		l = a;
		h = b;
		v = c;
	    }
	}
	public String toString() {
	    StringBuffer sb = new StringBuffer();
	    sb.append("Low key: ").append(l);
	    sb.append(" Hi key: ").append(h);
	    sb.append(" Interval: ").append(v);

	    return sb.toString();
	}
    }
    //
    class myCanvas extends Canvas {
	private static final long serialVersionUID = 1L;

        public void update (Graphics g) {
            can.paint(g);
        }
        public void paint(Graphics g) {
            paintI();
            g.drawImage(ig, 0, 0, this);
        }
    }
    public void paintI() {
        gi.setColor(Color.black);
        gi.fillRect(0, 0, W, H);

	DKEY = (double)W/KEY_BOARD;
	DOT = FONT/3;
	DSF = DOT/2;

        switch (GRPH) {
        case INHA:
            dispInha(gi);
            break;
        case CENT:
            dispCent(gi);
            break;
	case BEAT1:
	    dispBeat(gi);
	    break;
	case BEAT2:
	    dispBeatFlat(gi);
	    break;
	case BEAT0:
	    dispBeat0(gi);
	    break;
	case BEAT3:
	    dispBeatPart(gi);
	    break;
	default:
        }
    }
    public String getKeyname(int num) {
        return (num+1)+"("+getKey(num)+")";
    }
    public String getKey(int key) {
        return (key < 0) ? "" : KEY_NAME[key%OCTAVE];
    }
    //
    public void dispInha(Graphics g) {
	double byh = H/5.0;
	disp_log(g, byh);
	disp_aline(g);

	for (int i = 0; i < KEY_BOARD; i++) {
	    mouse_color(g, i);
            g.fillOval((int)(i*DKEY),
                       (int)(H2-log_base(MI.getInha(i))*byh-DSF), DOT, DOT);
        }
    }
    public void disp_log(Graphics g, double byh) {
        double h0 = 0.001, hl;
        int hm;

        for (int i = 1; i < 6; i++) {
            hl = h0*Math.pow(10, i);
            hm = (int)(H2-byh*log_base(hl));

            g.setColor(Color.blue);
            g.drawLine(0, hm, W-1, hm);

            g.setColor(Color.orange);
            g.drawString(""+(float)hl, 2, hm);
        }
    }
    public void disp_aline(Graphics g) {
        int wi;

        for (int i = 0; i < OCTAVE_NAME.length; i++) {
	    wi = (int)(DKEY*i*OCTAVE+DSF);

	    g.setColor(Color.blue);
	    g.drawLine(wi, 0, wi, H);

	    g.setColor(Color.red);
	    g.drawString(OCTAVE_NAME[i], wi, FONT*2);
        }
    }
    public void mouse_color(Graphics g, int key) {
	if (key == KEY)
	    g.setColor(Color.green);
	else if (key > WOUND)
	    g.setColor(Color.white);
	else
	    g.setColor(Color.pink);
    }
    //
    public void dispCent(Graphics g) {
	double dh = (double)H2/CENT_MAX;
	disp_horizontal_line(g, dh);
	disp_aline(g);

	for (int i = 0; i < KEY_BOARD; i++) {
	    mouse_color(g, i);
            g.fillOval((int)(i*DKEY),
		       (int)(H2-TT.getCent(i)*dh-DSF), DOT, DOT);
        }
    }
    public void disp_horizontal_line(Graphics g, double dh) {
        int hp, hm, xs = 2, hx = CENT_MAX/5;
        double hy, hi;

        disp_center_line(g);

        for (int i = 1; i < 6; i++) {
            hi = hx*i;
            hy = hi*dh;
            hp = (int)(H2+hy);
            hm = (int)(H2-hy);

            g.setColor(Color.blue);
            g.drawLine(0, hm, W, hm);
            g.drawLine(0, hp, W, hp);

            g.setColor(Color.yellow);
            g.drawString(""+hi, xs+2, hm+FONT);
            g.drawString("-"+hi, xs, hp);
        }
    }
    public void disp_center_line(Graphics g) {
        g.setColor(Color.gray);
        g.drawLine(0, H2, W, H2);
    }
    //
    public void dispBeat(Graphics g) { // beat1
	drawCircle(g);

	DOT = FONT/2;
	DSF = DOT/2;

	int ang = -90;
	Interv iv = new Interv();
	Point p0;
        for (int i = 0; i < OCTAVE; i++) { // interval
	    iv.setInterv(IN.getInterval(i), LR_SW);
	    draw_inte_name(g, ang, i);

	    if (INTV[i]) {
		for (int j = MULTI; j > 0; j--) {
		    p0 = calc_point(g, j, iv, ang, Col[i]);
		    g.fillRect(p0.x-DSF, p0.y-DSF, DOT, DOT);
		}
	    }
	    ang += 30;
        }
    }
    public Point calc_point(Graphics g, int j, Interv iv, int ang, Color col) {
	double gin = calc_sindo2(TT.get_ifreq(KEY, j*iv.l), j*iv.l,
				 TT.get_ifreq(KEY+iv.v, j*iv.h), j*iv.h);
	g.setColor(calcColor(gin, col));
	double be = Math.abs(getMbeats2(KEY, iv, j))*BBY;

	return point_circle(ang, be, W2, H2);
    }
    //
    public double getMbeats2(int key, Interv iv, int mul) {
	int keyh = key+iv.v;
	return get_mbeats2(keyh, key, iv, mul);
    }
    public double get_mbeats2(int keyh, int keyl, Interv iv, int mul) {
        return get_ibeats(keyh, keyl, calc_multi2(iv, mul));
    }
    public int[] calc_multi2(Interv iv, int mul) {
        int[] rat = new int[3];

        if (mul < 1) {
            System.out.println("*** Multi input error. ***");
            mul = 1;
        }
        rat[0] = iv.l*mul; // low
        rat[1] = iv.h*mul; // hi
        rat[2] = iv.v;

        return rat;
    }
    public double get_ibeats(int keyh, int keyl, int[] ratio) {
        double fh = TT.get_ifreq(keyh, ratio[1]);
        double fl = TT.get_ifreq(keyl, ratio[0]);

        return (fh-fl);
    }
    //
    public Color calcColor(double val, Color col) {
        float sft = 0.25f;
        float[] c0 = new float[3];
        Color.RGBtoHSB(col.getRed(), col.getGreen(), col.getBlue(), c0);

        c0[2] = (float)(c0[2]*val*COLOR_BY+sft);
        if (c0[2] > 1.0f)
            c0[2] = 1.0f;
        Color hsb = Color.getHSBColor(c0[0], c0[1], c0[2]);

        return COLOR_SW ? col : hsb;
    }
    public double calc_sindo2(double fr0, int pa0, double fr1, int pa1) {
        double gin0 = calc_sindo(fr0, pa0);
	double gin1 = calc_sindo(fr1, pa1);

	return gin0*gin1;
    }
    public double calc_sindo(double freq, int pr) {
        double fg = get_sboard(freq);
        double sp = get_spoint();
        double ga = spoint(pr, sp);

        return ga*fg;
    }
    public double calc_sindo(double freq, int pr, int key) { // for beat0
        double fg = get_sboard(freq);
        double sp = get_spoint(key);
        double ga = spoint(pr, sp);

        return ga*fg;
    }
    public double calc_sindo2(double fr0, int pr0, int key0,
			      double fr1, int pr1, int key1) {
	double gin0 = calc_sindo(fr0, pr0, key0);
	double gin1 = calc_sindo(fr1, pr1, key1);

	return gin0*gin1;
    }
    public double get_spoint() { // 0~48:1/8
        return (KEY <= 48) ? (double)SP49 : SP49*Math.pow(SAQ, KEY-48);
    }
    public double get_spoint(int key) {  // for beat0
        return (key <= 48) ? (double)SP49 : SP49*Math.pow(SAQ, key-48);
    }
    public double spoint(int mm, double sp) {
        double mp = mm * Math.PI;
        return Math.abs(8.0*Math.sin(mp/sp)/mp);
    }
    public double get_sboard(double freq) {
        double CutoffH = 170.0;
        double CutoffL = 20.0;
        freq -= CutoffL;
        double gain = freq/CutoffH;

        if (gain < 0)
            gain = 0.0;
        else if (gain > 1.0)
            gain = 1.0;

        return gain;
    }
    public void draw_inte_name(Graphics g, int ang, int num) {
	Point p = point_circle(ang, W2, W2, H2);

	g.setColor(Color.lightGray);
	if (num == 0)       // octave
	    p.y += FONT;
	else if (num == 3)  // mi.3rd
	    p.x -= 40;
	else if (num == 2 | num == 4) // Mj.2nd/Mj.3rd
	    p.x -= 20;

	g.drawString(IN.getInter(num), p.x, p.y);
    }
    public void drawCircle(Graphics g) {
	draw_spoke(g);
	draw_circle(g, DSF);

	double r = Math.min(W2/BBY, H2/BBY);
	draw_circle_level(g, r);
	circle_level(g, r);
	draw_circle_level(g, r/2);
	circle_level(g, r/2);
    }
    public void draw_spoke(Graphics g) {
	Point p0, p1;
	int ang = -90;

	g.setColor(Color.gray);
        for (int i = 0; i < OCTAVE; i++) {
	    p0 = point_circle(ang, DSF/2, W2, H2);
	    p1 = point_circle(ang, W, W2, H2);

	    g.drawLine(p0.x, p0.y, p1.x, p1.y);
	    ang += 30;
	}
    }
    public void draw_circle_level(Graphics g, double r) {
	Point p0, p1 = new Point(0,0);
	int ang = -90;

	g.setColor(Color.gray);
	for (int i = 0; i <= OCTAVE; i++) {
	    p0 = point_circle(ang, r*BBY, W2, H2);
	    if (i > 0)
		g.drawLine(p0.x, p0.y, p1.x, p1.y);

	    p1 = p0;
	    ang += 30;
	}
    }
    public void circle_level(Graphics g, double lev) {
	g.setColor(Color.red);
	g.drawString(dform0(lev), (int)(W2-lev*BBY), H2+FONT);
    }
    public void draw_circle(Graphics g, int r) {
	double r0 = r/100.0;

	g.setColor(Color.gray);
	g.drawOval((int)(W2*(1-r0)), (int)(H2*(1-r0)),
		   (int)(W*r0), (int)(H*r0));
    }
    public Point point_circle(int ang, double r, double x, double y) {
	Point p = new Point((int)((r+DSF)*Math.cos(RAD*ang)+x),
			    (int)((r+DSF)*Math.sin(RAD*ang)+y));

        return p;
    }
    //
    public void dispBeatFlat(Graphics g) { // beat2
	double dy = (H-FONT)/(double)INTN;
	Interv iv = new Interv();

	draw_b2_line(g, dy);
	draw_b2_cent(g);

	int cnt = 0;
	for (int i = 0; i < OCTAVE; i++) {
	    if (INTV[i]) {
		iv.setInterv(IN.getInterval(i), LR_SW);
		draw_b2_name(g, i, cnt, dy);

		if (LR_SW && KEY+iv.v >= 0)
		    draw_beat2(g, cnt, iv, dy, Col[i]);
		else if (!LR_SW && KEY+iv.v < KEY_BOARD)
		    draw_beat2(g, cnt, iv, dy, Col[i]);

		cnt++;
	    }
	}
    }
    public void draw_beat2(Graphics g, int cnt, Interv iv, 
			   double dy, Color col) {
	int yy, ce = 50;
	double dx = W/(ce*2.0);
	double be, gin, ctmp = TT.getCent(KEY);

	for (int c = -ce; c <= ce ; c++) {
	    TT.setCent(KEY, ctmp+c/WBY);

	    for (int j = MULTI; j > 0; j--) {
		be = Math.abs(getMbeats2(KEY, iv, j));
		yy = (int)(cnt*dy+BBY*be+FONT-DSF);
		if (yy > (cnt+1)*dy+FONT)
		    continue;

		gin = calc_sindo2(TT.get_ifreq(KEY, j*iv.l), j*iv.l,
				      TT.get_ifreq(KEY+iv.v, j*iv.h), j*iv.h);
		g.setColor(calcColor(gin, col));
		if (LR_SW)
		    g.fillRect((int)(dx*(c+ce)-DSF), yy, DOT,DOT);
		else
		    g.fillOval((int)(dx*(c+ce)-DSF), yy, DOT,DOT);
	    }
	}

	TT.setCent(KEY, ctmp);
    }
    public void draw_b2_line(Graphics g, double dy) {
	g.setColor(Color.gray);
	g.drawLine(W2, 0, W2, H);

	int y0;
	for (int i = 0; i < INTN; i++) {
	    y0 = (int)(FONT+dy*i);
	    g.drawLine(0, y0, W, y0);
	}

	g.setColor(Color.red);
	g.drawString(dform0(dy/BBY), W2, H);
    }
    public void draw_b2_name(Graphics g, int i, int cnt, double dy) {
	g.setColor(Color.lightGray);
	g.drawString(IN.getInter(i), W-40, (int)(cnt*dy+FONT*2));
    }
    public void draw_b2_cent(Graphics g) {
	g.setColor(Color.gray);
	g.drawLine(0, FONT, W, FONT);

	g.setColor(Color.white);
	g.drawString("-"+dform0(CENT_WIDTH), 0, FONT);
	g.drawString(dform0(CENT_WIDTH), W-20, FONT);
    }
    //
    public void dispBeat0(Graphics g) { // beat3
	disp_beat0_line(g);
	disp_aline(g);
	if (!ABS_SW)
	    disp_center_line(g);

	int h2 = (ABS_SW) ? H : H2;
	int start, end, be;
	double db = h2/BEAT_MAX;
	Interv iv = new Interv();

	for (int i = 0; i < OCTAVE; i++) {
	    if (INTV[i]) {
		iv.setInterv(IN.getInterval(i), LR_SW);
		start = (LR_SW) ? -iv.v : 0;
		end = (LR_SW) ? KEY_BOARD : KEY_BOARD-iv.v;

		for (int k = start; k < end; k++) {
		    for (int j = MULTI; j > 0; j--) {
			be = calc_beat0(g, i, k, j, iv, h2, db);
			if (be <= 0 || be > H)
			    continue;

			if (LR_SW)
			    g.fillRect((int)(k*DKEY), be, DOT, DOT);
			else
			    g.fillOval((int)(k*DKEY), be, DOT, DOT);
		    }
		}
	    }
	}
    }
    public int calc_beat0(Graphics g, int i, int k, int j,
			  Interv iv, int h2, double db) {
	double gin = calc_sindo2(TT.get_ifreq(k, j*iv.l), j*iv.l, k,
				 TT.get_ifreq(k+iv.v, j*iv.h), j*iv.h, k+iv.h);
	mouse_beat0_color(g, k, calcColor(gin, Col[i]));

	double be = getMbeats2(k, iv, j);
	be = (ABS_SW) ? Math.abs(be) : be;
	int b0 = (int)(h2-be*db-DSF);

	return b0;
    }
    public void mouse_beat0_color(Graphics g, int key, Color col) {
	g.setColor((key == KEY) ? Color.white : col);
    }
    public void disp_beat0_line(Graphics g) {
	int hm, hp = 0, max = 5;
	int h2 = (ABS_SW) ? H : H2;
	double hi, hv, db = BEAT_MAX/max, dh = h2/(double)max;

	for (int i = 1; i < 6; i++) {
            hv = i*db;
            hi = i*dh;
	    hm = (int)(h2-hi);
	    if (!ABS_SW)
		hp = (int)(h2+hi);

	    g.setColor(Color.blue);
            g.drawLine(0, hm, W, hm);
	    if (!ABS_SW)
		g.drawLine(0, hp, W, hp);

	    g.setColor(Color.yellow);
            g.drawString(dform0(hv), 4, hm+FONT);
	    if (!ABS_SW)
		g.drawString("-"+dform(hv), 2, hp);
	}
    }
    //
    public void dispBeatPart(Graphics g) { // beat4
	int dot = (int)(W/PARTIAL/2.0);
        int dsf = dot/2;
        int h2 = ABS_SW ? H : H2;
	int start, end, val;
        double be, gin;
	double dp = W/(double)MULTI;
	double dp2 = dp/2;
        double db = h2/BEAT_MAX;
	Interv iv = new Interv();

        disp_beatp_vline(g, dp, dp2, dsf);
        disp_beat0_line(g);
	if (!ABS_SW)
	    disp_center_line(g);

	for (int i = 0; i < OCTAVE; i++) {
	    if (INTV[i]) {
		iv.setInterv(IN.getInterval(i), LR_SW);
		start = (LR_SW) ? -iv.v : 0;
		end = (LR_SW) ? KEY_BOARD : KEY_BOARD-iv.v;
		if (KEY < start || KEY >= end)
		    continue;

		for (int j = 1; j <= MULTI; j++) {
		    be = getMbeats2(KEY, iv, j);
		    be = ABS_SW ? Math.abs(be) : be;
		    val = (int)(h2-be*db-dsf);
		    if (val <= 0 || val > H) // <=
			continue;

		    gin = calc_sindo2(TT.get_ifreq(KEY, j*iv.l), j*iv.l, KEY,
				      TT.get_ifreq(KEY+iv.v, j*iv.h),
				      j*iv.h, KEY+iv.v);
		    g.setColor(calcColor(gin, Col[i]));
		    if (LR_SW)
			g.fillRect((int)(dp*j-(dp2+dsf)), val, dot, dot);
		    else
			g.fillOval((int)(dp*j-(dp2+dsf)), val, dot, dot);
		}
	    }
	}
    }
    public void disp_beatp_vline(Graphics g, double dp, double dp2, int dsf) {
	int dx;

	for (int i = 1; i <= MULTI; i++) {
	    g.setColor(Color.blue);
	    dx = (int)(dp*i-dp2);
	    g.drawLine(dx, 0, dx, H);

	    g.setColor(Color.red);
	    g.drawString(""+i, dx, FONT*2);
	}
    }
}
